"""An Editable Description content type"""

__author__  = 'Daniel J. Vreeman, PT, DPT'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.EditableDescription.interfaces import IEditableDescription

from Products.ATContentTypes.content.newsitem import ATNewsItem
from Products.ATContentTypes.content.newsitem import finalizeATCTSchema
from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.schemata import *
from Products.ATContentTypes.content.base import updateActions, updateAliases
from Products.ATContentTypes.content.base import ATCTContent

from Products.ATContentTypes.configuration import zconf

from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from ComputedAttribute import ComputedAttribute

from Products.validation.config import validation
from Products.validation.validators.SupplValidators import MaxSizeValidator
from Products.validation import V_REQUIRED

from Products.EditableDescription.Permissions import *
from Products.EditableDescription.Permissions import EditEditableDescriptionPermission
from Products.EditableDescription.Permissions import EditEditableDescriptionOnlyPermission

from Products.CMFCore.permissions import View

validation.register(MaxSizeValidator('checkNewsImageMaxSize',
                                     maxsize=zconf.ATNewsItem.max_file_size))




# Copy the ATDocument schema (to avoid modifying the original) and append our
# own fields for the images and attachments manager widgets.

FirstEditableDescriptionSchema = ATDocument.schema.copy()
del FirstEditableDescriptionSchema['title']
del FirstEditableDescriptionSchema['description']
del FirstEditableDescriptionSchema['text']
del FirstEditableDescriptionSchema['relatedItems']
del FirstEditableDescriptionSchema['allowDiscussion']
del FirstEditableDescriptionSchema['excludeFromNav']
del FirstEditableDescriptionSchema['rights']

EditableDescriptionSchema = FirstEditableDescriptionSchema + Schema((

    StringField('title',
        required=1,
        searchable=1,
        default='',
        accessor='getTitle',
        write_permission='EditableDescription: Edit EditableDescription',
        widget=StringWidget(
            label="Term or Part",
            label_msgid="label_term",
            description = "LOINC Term or Part Number.",
            description_msgid="help_term"),
    ),

   StringField('description',
        required=1,
        searchable=1,
        default='',
        accessor='getDescription',
        write_permission='EditableDescription: Edit EditableDescription',
        widget=TextAreaWidget(
            label="LOINC Name",
            label_msgid="label_description",
            description = "Formal LOINC name of term or part.",
            description_msgid="help_description"),
    ),

    TextField('text',
        required = False,
        searchable = True,
        primary = True,
        accessor='getText',
        write_permission='EditableDescription: Edit EditableDescription',
        storage = AnnotationStorage(migrate=True),
        validators = ('isTidyHtmlWithCleanup',),
        default_output_type = 'text/x-html-safe',
        default_content_type = zconf.ATDocument.default_content_type,
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            description = "LOINC Definition",
            description_msgid = "help_additional_description",
            label = "LOINC Definition. From COMMENTS field.",
            label_msgid = "label_additional_description",
            rows = 3,
            i18n_domain = "plone",
            allow_file_upload = zconf.ATDocument.allow_document_upload)
        ),

    TextField('auxDescription1Text',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        default_output_type = 'text/plain',
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain'),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=TextAreaWidget(
            label="Auxiliary Description 1 Text",
            label_msgid="label_auxdescription1text",
            description = "Text of auxiliary description #1.",
            description_msgid="help_auxdescription1text"),
    ),

    StringField('auxDescription1Source',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=StringWidget(
            label="Source for description",
            label_msgid="label_auxDescription1Source",
            description = "Source for description.",
            description_msgid="help_auxDescription1Source"),
    ),

    StringField('auxDescription1URL',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        validators = ('isURL',),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=StringWidget(
            label="URL for description",
            label_msgid="label_auxDescription1URL",
            description = "Full URL link for description. Include the http:// prefix.",
            description_msgid="help_auxDescription1URL"),
    ),

        TextField('auxDescription2Text',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        default_output_type = 'text/plain',
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain'),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=TextAreaWidget(
            label="Auxiliary Description 2 Text",
            label_msgid="label_auxdescription1text",
            description = "Text of auxiliary description #2.",
            description_msgid="help_auxdescription2text"),
    ),

    StringField('auxDescription2Source',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=StringWidget(
            label="Source for description",
            label_msgid="label_auxDescription2Source",
            description = "Source for description.",
            description_msgid="help_auxDescription2Source"),
    ),

    StringField('auxDescription2URL',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        validators = ('isURL',),
        write_permission='EditableDescription: Edit EditableDescription',
        widget=StringWidget(
            label="URL for description",
            label_msgid="label_auxDescription2URL",
            description = "Full URL link for description. Include the http:// prefix.",
            description_msgid="help_auxDescription2URL"),
    ),

    TextField('communityDescription',
        required=0,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        default_output_type = 'text/plain',
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain'),
        mutator='setMyCommunityDescription',
        write_permission='EditableDescription: Edit Description Only for EditableDescription',
        widget=TextAreaWidget(
            label="Community-contributed Description",
            label_msgid="label_communitydescription",
            description = "Add and edit the community-contributed description. Only plain text is stored.",
            description_msgid="help_communitydescription"),
    ),

    ReferenceField('relatedItems',
        relationship = 'relatesTo',
        multiValued = True,
        isMetadata = True,
        languageIndependent = False,
        index = 'KeywordIndex',
        write_permission='EditableDescription: Edit EditableDescription',
        widget = ReferenceBrowserWidget(
            allow_search = True,
            allow_browse = True,
            show_indexes = False,
            force_close_on_insert = True,

            label = "Related Item(s)",
            label_msgid = "label_related_items",
            description = "",
            description_msgid = "help_related_items",
            i18n_domain = "plone",
            visible = {'edit' : 'visible', 'view' : 'invisible' }
            ),),

        StringField(
            'allowDiscussion',
            accessor="isDiscussable",
            mutator="allowDiscussion",
            edit_accessor="editIsDiscussable",
            default=None,
            write_permission='EditableDescription: Edit EditableDescription',
            enforceVocabulary=1,
            vocabulary=DisplayList((
        ('None', 'Default', 'label_discussion_default'),
        ('1',    'Enabled', 'label_discussion_enabled'),
        ('0',    'Disabled', 'label_discussion_disabled'),
        )),
            widget=SelectionWidget(
                label="Allow Discussion on this item",
                label_msgid="label_allow_discussion",
                description_msgid="help_allow_discussion",
                i18n_domain="plone"),
        ),
         BooleanField('excludeFromNav',
         required = False,
         languageIndependent = True,
         default = True,
         schemata = 'metadata', # moved to 'default' for folders
         widget = BooleanWidget(
             description="If selected, this item will not appear in the navigation tree",
             description_msgid = "help_exclude_from_nav",
             label = "Exclude from navigation",
             label_msgid = "label_exclude_from_nav",
             i18n_domain = "plone",
             visible={'view' : 'hidden',
                      'edit' : 'visible'},
             ),
        ),
        
        TextField(
            'rights',
            accessor="Rights",
            default = 'Copyright  1995-2008 Regenstrief Institute and the LOINC Committee. All rights reserved.',
            schemata = 'metadata',
            widget=TextAreaWidget(
                label='Copyrights',
                description="The copyrights on this item.",
                label_msgid="label_copyrights",
                description_msgid="help_copyrights",
                i18n_domain="plone")),

    ),)

# Finalise the schema according to ATContentTypes standards. This basically
# moves the Related items and Allow discussion fields to the bottom of the
# form. See ATContentTypes.content.schemata for details.

finalizeATCTSchema(EditableDescriptionSchema)

# Declare our class to get all the fields from ATDocument.

class EditableDescription(ATDocument):
    """
    An editable description
    """

    # Standard content type setup
    portal_type = meta_type = 'EditableDescription'
    archetype_name = 'Editable Description'
    content_icon = 'EditableDescription.gif'
    schema = EditableDescriptionSchema
    typeDescription= 'An editable description'
    typeDescMsgId  = 'EditableDescription_description_edit'

    # Set up our views - these are available from the 'display' menu
    default_view = 'EditableDescription_view'
    immediate_view = 'EditableDescription_view'

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Make sure we get all the interface declarations from ATDocument,
    __implements__ = ATDocument.__implements__,(IEditableDescription, )

     # Get the standard actions (tabs)
     # Removed this to customize the permissions
     #actions = ATDocument.actions

    actions = updateActions(ATCTContent,
            (
                {
                'id': 'view',
                'permissions': (ViewEditableDescriptionPermission,),
                'action': 'string:${object_url}/view',
                },
                {
                'id': 'edit',
                'permissions': (EditEditableDescriptionOnlyPermission,),
                'action': 'string:${object_url}/EditableDescription_edit',
                },
                                

                # Hide properties tab from ordinary users
                { 'id': 'metadata',
                  'name': 'Properties',
                  'action': 'string:${object_url}/base_metadata',
                  'permissions': (EditEditableDescriptionPermission,),
                },   
                
                {
                  'id'          : 'local_roles',
                  'name'        : 'Sharing',                  
                  'permissions' : (EditEditableDescriptionPermission,),
                }, 
                                {
                  'id'          : 'edit_master',
                  'name'        : 'Master Edit',
                  'permissions' : (EditEditableDescriptionPermission,),
                  'action': 'string:${object_url}/edit',
                },
            )
        )


    # This method, from ISelectableBrowserDefault, is used to check whether
    # the "Choose content item to use as default view" option will be
    # presented.
    def canSetDefaultPage(self):
        return False

    # enable FTP/WebDAV and friends
    PUT = ATDocument.PUT

    # Stuff from News Item ... dont' know if I need it or not.

    security = ClassSecurityInfo()
    
    # My attempt at a permission customized mutator
    
    security.declareProtected(EditEditableDescriptionOnlyPermission, 'setMyCommunityDescription')
    def setMyCommunityDescription(self, value):
        self.getField('communityDescription').set(self, value)
    
    # Remix of function to get contributors. Returns a string rather than a list.
    
    security.declareProtected(ViewEditableDescriptionPermission, 'getContributorsString')
    def getContributorsString(self):
        """Dublin Core element - Contributors"""
        tupleContribs = self.Contributors()
        stringContribs = ','.join(tupleContribs)
        return stringContribs


registerType(EditableDescription)
