######################################################################
#
# Extension Rename - Hook into Archetypes to enforce extension usage.
# Copyright(C), 2005, Enfold Systems, Inc. - ALL RIGHTS RESERVED
#
# This software is licensed under the Terms and Conditions contained
# within the LICENSE.txt file that accompanied this software.  Any
# inquiries concerning the scope or enforceability of the license should
# be addressed to:
#
# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
######################################################################
"""
$Id: monkey.py 5635 2006-07-03 15:44:24Z sidnei $
"""

from Products.Archetypes.utils import shasattr
from Products.CMFCore.utils import getToolByName
from Products.ExtensionRename.config import log, log_exc

def renameMethod(file_ext):
    """Method factory to create renameAfterCreation methods that use
    different file extensions, yet do basically the same thing.

    Takes non-dotted file extension.

    Returns method suitable for monkey patching as '_renameAfterCreation'
    """
    fext = ".%s" % file_ext


    def _renameAfterCreation(self, check_auto_id=False):
        """Rename objects like their titles, including file extension.
        File extension for this method is .%s
        """ % fext
        plone_tool = getToolByName(self, 'plone_utils', None)
        if plone_tool is None or not shasattr(plone_tool, 'normalizeString'):
            # Plone tool is not available or too old
            # XXX log?
            return None

        title = self.Title()
        if not title:
            # Can't work w/o a title
            return False

        old_id = self.getId()
        if check_auto_id and not self._isIDAutoGenerated(old_id):
            # No auto generated id
            return False

        new_id = plone_tool.normalizeString(title)
        if not new_id.endswith(fext):
            # add extension if not already there
            new_id = new_id + fext
        invalid_id = False

        check_id = getattr(self, 'check_id', None)

        if check_id is not None:
            invalid_id = check_id(new_id, required=1)
        else:
            # If check_id is not available just look for conflicting ids
            parent = aq_parent(aq_inner(self))
            invalid_id = new_id in parent.objectIds()

        if not invalid_id:
            # Can't rename without a subtransaction commit when using
            # portal_factory!
            get_transaction().commit(1)
            self.setId(new_id)
            return new_id

        return False

    return _renameAfterCreation

renameHTML = renameMethod('html')
renameICS = renameMethod('ics')
renameURL = renameMethod('url')

try:
    from Products.ATContentTypes.atct import ATDocument, ATEvent, ATLink
    ATDocument._renameAfterCreation = renameHTML
    ATEvent._renameAfterCreation = renameICS
    ATLink._renameAfterCreation = renameURL
    log('Hooked ATContentTypes to forcibly add extensions '
        'to newly-created Document, Event and Link.')
except ImportError:
    log_exc('ATContentTypes not available, not hooking extensions.')
