##############################################################################
#
# Lime - Link Marshaller
# Copyright(C), 2004-5, Enfold Systems, Inc. - ALL RIGHTS RESERVED
#
# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
##############################################################################
"""
$Id: test_doctest.py 5635 2006-07-03 15:44:24Z sidnei $
"""


import os, sys
from os import curdir
from os.path import join, abspath, dirname, split
from StringIO import StringIO
from dateutil.tz import tzutc

try:
    __file__
except NameError:
    # Test was called directly, so no __file__ global exists.
    _prefix = abspath(curdir)
else:
    # Test was called by another test.
    _prefix = abspath(dirname(__file__))

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
import unittest
from Testing import ZopeTestCase
from Testing.ZopeTestCase import doctest
from Testing.ZopeTestCase import FunctionalDocFileSuite as DocSuite

# Install our product
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('Lime')
ZopeTestCase.installProduct('ATContentTypes')

from Products.CMFPlone.tests import PloneTestCase
from Products.Lime.tests.utils import installType

class TestATLink(PloneTestCase.FunctionalTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.addProduct('PortalTransforms')
        self.addProduct('Archetypes')

        installType(self.portal,
                    'ATLink',
                    'ATContentTypes',
                    'Link',
                    ignore_actions=True)

class TestCMFLink(PloneTestCase.FunctionalTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.addProduct('PortalTransforms')
        self.addProduct('Archetypes')

        # Install CMF Link as Link.
        pt = self.portal.portal_types
        if 'Link' in pt.objectIds():
            pt.manage_delObjects(ids=['Link'])
        import pdb;pdb.set_trace()
        pt.manage_addTypeInformation(
            id='Link',
            add_meta_type='Factory-based Type Information',
            typeinfo_name='CMFDefault: Link (Link)')

class TestATFavorite(PloneTestCase.FunctionalTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.addProduct('PortalTransforms')
        self.addProduct('Archetypes')

        installType(self.portal,
                    'ATFavorite',
                    'ATContentTypes',
                    'Favorite',
                    ignore_actions=True)

def test_suite():
    suite = unittest.TestSuite()
    functional = [
        (TestATLink, 'export.txt'),
        # (TestCMFLink, 'export.txt'),
        (TestATFavorite, 'favorite.txt'),
        ]
    for test, f in functional:
        suite.addTest(DocSuite(f, test_class=test,
                               package='Products.Lime.tests',
                               globs=globals()))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
