import string
from DateTime import DateTime

from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
# needed for getAnonymousID
from Products.CMFCore.utils import getToolByName

from Products.MeetingRegistration.config import REGISTRATION_STATUS, NOTIFICATION_METHOD
from Products.MeetingRegistration.Permissions import *

HeaderSchema = BaseSchema.copy()
del HeaderSchema['title']

schema = HeaderSchema + Schema((

    StringField('title',
        required=1,
        searchable=1,
        default='',
        accessor='getTitle',
        widget=StringWidget(
            label="Title",
            label_msgid="label_title",
            description = "Enter a title.",
            description_msgid="help_title"),
    ),

    StringField('registrationStatus',
        searchable=0,
        required=0,
        default="open",
        vocabulary=REGISTRATION_STATUS,
        widget=SelectionWidget(
            label="Registration currently",
            label_msgid="label_question_description",),
        ),

    TextField('thankYouMessage',
        required=0,
        searchable=0,
        default="""Thank you for completing the registration.""",
        widget=TextAreaWidget(
            label="'Thank you' message text",
            label_msgid="label_thanku",
            description="""This is the message that will be displayed to the
                           user when they complete the registration.""",
            description_msgid="help_thanku"),
        ),

    TextField('savedMessage',
        required=0,
        searchable=0,
        default="""You have saved the registration.
Don't forget to come back and finish it.""",
        widget=TextAreaWidget(
            label="'Saved' message test",
            label_msgid="label_saved_text",
            description="""This is the message that will be displayed to the user
                           when they save the registration, but don't submit it.""",
            description_msgid="label_saved_text"),
        ),

    StringField('exitUrl',
        required=0,
        searchable=0,
        widget=StringWidget(
            label="Exit URL",
            label_msgid="label_exit_url",
            description="""This is the URL that the user will be directed to on completion of the registration.
                           Use "http://site.to.go.to/page" or "route/to/page" for this portal""",
            description_msgid="help_exit_url"),
        ),

    BooleanField('allowAnonymous',
        searchable=0,
        required=0,
        widget=BooleanWidget(
            label="Allow Anonymous",
            label_msgid="label_allow_anonymous",),
        ),

    StringField('registrationNotificationEmail',
        required=0,
        searchable=0,
        widget=StringWidget(
            label="Registration Notification Email Address",
            label_msgid="label_registration_notification_email",
            description="Enter an email address to receive notifications of registration completions.",
            description_msgid="help_registration_notification_email"),
        ),

    StringField('registrationNotificationMethod',
        required=0,
        searchable=0,
        vocabulary=NOTIFICATION_METHOD,
        widget=SelectionWidget(
            label="Registration Notification Method",
            label_msgid="label_registration_notification_method",
            description="Select a method to receive notification emails.",
            description_msgid="help_registration_notification_method"),
        ),

    StringField('completedFor',
        searchable=0,
        required=0,
        default=[],
        widget=StringWidget(visible=0,),
        ),

    ))

Actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'string:${object_url}/registration_view',
    'permissions' : (VIEW_CONTENTS_PERMISSION, )
    },
    {
    'id' : 'results',
    'name' : 'Review results',
    'action' : 'string:${object_url}/registration_view_results',
    'permissions' : (EDIT_CONTENTS_PERMISSION, )
    },
    {
    'id' : 'reset',
    'name' : 'Reset',
    'action' : 'string:${object_url}/registration_reset_form',
    'permissions' : (EDIT_CONTENTS_PERMISSION, )
    },
)

class Registration(OrderedBaseFolder):
    """You can add questions to registrations"""
    schema = schema
    actions = Actions
    immediate_view = "base_edit"
    global_allow = 1
    filter_content_types = 1
    allowed_content_types = (['Registration Question', 'Sub Registration'])
    archetypes_name = portal_type = "Registration"
    meta_type = "Registration"
    content_icon = "icon_meetingregistration_small.gif"
    include_default_actions = 1
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'Title')
    def Title(self, **kw):
        """Override DC Title to include its status: open or closed"""
        title = self.getField('title').get(self)
        if self.getField('registrationStatus').get(self) =='open':
            return title + ' (open)'
        else:
            return title + ' (closed)'

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTitle')
    def getTitle(self):
        """return the title, or raw title if in edit form"""
        # check if in base_edit
        path_info = self.REQUEST.PATH_INFO
        if path_info.split('/')[-1:] == ['base_edit']:
            return self.getField('title').get(self)
        return self.Title()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isMultipage')
    def isMultipage(self):
        """Return true if there is more than one page in the registration"""
        if self.getFolderListingFolderContents(contentFilter={'portal_type':'Sub Registration',}):
            return True

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getQuestions')
    def getQuestions(self):
        """Return the questions for this part of the registration"""
        questions = self.getFolderListingFolderContents(contentFilter={'portal_type':'Registration Question',})
        return questions

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAllQuestions')
    def getAllQuestions(self):
        """Return all the questions in the registration"""
        portal_catalog = getToolByName(self, 'portal_catalog')
        questions = []
        path = string.join(self.getPhysicalPath(), '/')
        results = portal_catalog.searchResults(portal_type = 'Registration Question',
                                               path = path)
        for result in results:
            questions.append(result.getObject())
        return questions

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNextPage')
    def getNextPage(self):
        """Return the next page of the registration"""
        pages = self.getFolderListingFolderContents(contentFilter={'portal_type':'Sub Registration',})
        if pages:
            return pages[0]()
        # no next page, so registration finished
        userid = self.getCurrentId()
        self.setCompletedForUser(userid)
        return self.exitRegistration()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNextPage')
    def exitRegistration(self):
        """Return the defined exit url"""
        exit_url = self.getExitUrl()
        if exit_url[:7] != 'http://':
            exit_url = self.portal_url() + '/' + exit_url + '?portal_status_message=' + self.getThankYouMessage()
        return self.REQUEST.RESPONSE.redirect(exit_url)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNextPage')
    def setCompletedForUser(self, userid):
        """Set completed for a user"""
        completed = self.getCompletedFor()
        completed.append(userid)
        self.setCompletedFor(completed)
        self.notify_registrant(userid)


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'checkCompletedFor')
    def checkCompletedFor(self, user_id):
        """Check whether a user has completed the registration"""
        completed = self.getCompletedFor()
        if user_id in completed:
            return True
        return False

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'canViewQuestions')
    def canViewQuestions(self):
        """Check if user can view registration"""
        #copied from skins, does not work yet
        if context.portal_membership.isAnonymousUser() and not context.getAllowAnonymous():
            return 0
        if getattr(context, 'status', 'open') == 'closed':
            return 0
        return not context.checkCompletedFor(user_id)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getRegistrationId')
    def getRegistrationId(self):
        """Return the userid for the registration"""
        portal_membership = getToolByName(self, 'portal_membership')
        request = self.REQUEST
        response = request.RESPONSE

        if not portal_membership.isAnonymousUser():
            current_user = portal_membership.getAuthenticatedMember().getId()
            if self.checkCompletedFor(current_user):
               if not hasattr(self, 'registration_id_no'):
                   self.registration_id_no = 0
               else:
                   self.registration_id_no += 1
               return current_user + str(self.registration_id_no)
            else:
                 return portal_membership.getAuthenticatedMember().getId()

        else:
             if self.getAllowAnonymous():
                  current_user = self.getAnonymousId()
                  return current_user

        registration_cookie = '__meetingregistration_name'
        response.setCookie(registration_cookie, registration_id, path='/')

        return registration_id

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAnonymousId')
    def getAnonymousId(self):
    # returns the id to use for an anonymous user
        portal_membership = getToolByName(self, 'portal_membership')
        if portal_membership.isAnonymousUser() and self.getAllowAnonymous():
            if not hasattr(self, 'registration_id_no'):
                self.registration_id_no = 0
            else:
                self.registration_id_no += 1
            return 'anonymous' + str(self.registration_id_no)
        else:
            # XXX raise unathorized error
            pass

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'setCurrentId')
    def setCurrentId(self, userid):
        self.current_id = userid

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCurrentId')
    def getCurrentId(self):
        return self.current_id

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getRespondents')
    def getRespondents(self):
        """Return a list of respondents"""
        questions = self.getAllQuestions()
        users = {}
        for question in questions:
            for user in question.getAnswers().keys():
                users[user] = 1
        return users.keys()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getRespondents')
    def getAnswersByUser(self, userid):
        """Return a set of answers by user id"""
        questions = self.getAllQuestions()
        answers = {}
        for question in questions:
            answer = question.getAnswerFor(userid)
            answer[question.getId()] = answer
        return answers

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getQuestionsCount')
    def getQuestionsCount(self):
        """Return a count of questions asked"""
        return len(self.questions)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'resetForUser')
    def resetForUser(self, userid):
        """Remove answer for a single user"""
        completed = self.getCompletedFor()
        if userid in completed:
            completed.remove(userid)
        self.setCompletedFor(completed)
        questions = self.getAllQuestions()
        for question in questions:
            question.resetForUser(userid)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'send_email')
    def send_email(self, userid):
        """ Send email to nominated address """
        properties = self.portal_properties.site_properties
        mTo = self.getRegistrationNotificationEmail()
        mFrom = properties.email_from_address
        mSubj = '[%s] New registration submitted' % self.Title()
        message = []
        message.append('Hi,')
        message.append('A registration has been completed by user: %s.' % userid)
        message.append(self.absolute_url() + '/registration_view_results')
        mMsg = '\n\n'.join(message)
        try:
            self.MailHost.send(mMsg, mTo, mFrom, mSubj)
        except:
            # XXX too many things can go wrong
            pass

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getEmailQuestion')
    def getEmailQuestion(self, userid):
        """Return the question for the registration that asks for an email address"""
        results = self.getFolderListingFolderContents(contentFilter={'portal_type':'Registration Question',})
        questions = []
        for result in results:
            if result.emailField:
               questions.append(result.getAnswerFor(userid))
        return questions

# Need to add get all answers function to call here.

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAllAnswers')
    def getAllAnswers(self, userid):
        """Return the question for the registration that asks for email address"""
        results = self.getFolderListingFolderContents(contentFilter={'portal_type':'Registration Question',})
        answers = []
        allAnswers = []
        for result in results:
            answers.append('%s: %s' % (result.getId(), result.getAnswerFor(userid)))
        allAnswers = '\n'.join(answers)
        return allAnswers

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'notify_registrant')
    def notify_registrant(self, userid):
        """ Send email to nominated address """
        properties = self.portal_properties.site_properties
        results = self.getAllAnswers(userid)
        mTo = self.getEmailQuestion(userid)
        if self.getRegistrationNotificationMethod() == 'each_submission':
             mTo.append(self.getRegistrationNotificationEmail())
        mFrom = self.getRegistrationNotificationEmail()
        mSubj = '[%s] Registration Confirmation' % self.Title()
        message = []
        message.append('Greetings,')
        message.append('Thank you for registering for the meeting: %s. You are registered for the following:' % self.Title())
        message.append('%s' % results)
        message.append('Kind Regards from the Regenstrief Institute, Inc.')
        mMsg = '\n\n'.join(message)
        try:
            self.MailHost.send(mMsg, mTo, mFrom, mSubj)
        except:
            pass
               #exit_url = self.portal_url() + '/' + exit_url + '?portal_status_message=Registrant Notification Failed'
               #return self.REQUEST.RESPONSE.redirect(exit_url)

registerType(Registration)