from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.MeetingRegistration.Permissions import *

from Products.MeetingRegistration.config import INPUT_TYPE, COMMENT_TYPE, BARCHART_COLORS

HeaderSchema = BaseSchema.copy()

schema = HeaderSchema + Schema((

    StringField('enablingObjective',
        searchable=0,
        required=0,
        widget=StringWidget(
            label="Enabling Objective",
            label_msgid="label_enable_objective",
            description="Enabling Objective code, in the correct format, e.g. EO35RF",
            description_msgid="help_enable_objective"),
        ),

    LinesField('answerOptions',
        searchable=0,
        required=0,
        default=("Yes", "No"),
        widget=LinesWidget(
            label="Answer options",
            label_msgid="label_answer_options",
            description="""Enter the options you want to be available to the user here.
                           This only applies to the selection box, multiple selection box,
                           checkbox and radio button input types (selected below).
                           The text input options allow the user to type the input
                           rather than selecting from options. Press enter to seperate
                           the options.""",
            description_msgid="help_answer_options"),
        ),

    StringField('inputType',
        searchable=0,
        required=0,
        vocabulary=INPUT_TYPE,
        widget=SelectionWidget(
            label="Input Type",
            label_msgid="label_input_type",
            description="Please select what type of input you would like to use for this question.",
            description_msgid="help_input_type"),
        ),

    BooleanField('required',
        searchable=0,
        required=0,
        default=1,
        widget=BooleanWidget(
            label="Required",
            label_msgid="label_required",
            description="Select if this question is required, meaning participant must give a response.",
            description_msgid="help_required"),
        ),

    StringField('commentType',
        searchable=0,
        required=0,
        vocabulary=COMMENT_TYPE,
        widget=SelectionWidget(
            label="Comment Type",
            label_msgid="label_comment_type",
            description="Select what type of comment box you would like.",
            description_msgid="help_label_comment_type",
            format="select"),
        ),

    StringField('commentLabel',
        searchable=0,
        required=0,
        default="Comment - mandatory if \"no\"",
        widget=TextAreaWidget(
            label="Comment label",
            label_msgid="label_comment_label",
            description="The comment label.",
            description_msgid="help_comment_label"),
        ),

    StringField('answers',
        searchable=0,
        required=0,
        default={},
        widget=StringWidget(visible=0,),
        ),

    BooleanField('emailField',
		default = False,
		searchable = False,
		widget = BooleanWidget(
			label = 'This question stores an email address ',
			description = 'Check this box if the question stores an email address',
		),
	),

    ))

class RegistrationQuestion(BaseContent):
    """A question within a registration"""
    schema = schema
    #actions = Actions
    immediate_view = "base_edit"
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ()
    archetypes_name = portal_type = "Registration Question"
    meta_type = "RegistrationQuestion"
    content_icon = "icon_meetingregistration_small.gif"
    include_default_actions = 1
    _at_rename_after_creation = True

    security = ClassSecurityInfo()


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'reset')
    def reset(self):
        """Remove all my answers"""
        self.setAnswers({})

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'resetForUser')
    def resetForUser(self, userid):
        """Remove answer for a single user"""
        answers = self.getAnswers()
        if answers.has_key(userid):
            del answers[userid]
        self.setAnswers(answers)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'addAnswer')
    def addAnswer(self, userid, value, comments=""):
        """Add an answer and optional comments for a user."""
        #answered = self.values_by_user.has_key(userid)
        answers = self.getAnswers()
        answers[userid] = {'value':value,
                           'comments':comments}
        self.setAnswers(answers)
        #if not answered:
        #    if not self.value_counts.has_key(value):
        #        self.value_counts[value] = 0
        #    self.value_counts[value] += 1
        #self._p_changed = 1

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAnswerFor')
    def getAnswerFor(self, userid):
        """Get a specific user's answer"""
        answer = self.answers.get(userid, {}).get('value', None)
        if self.getInputType() in ['multipleSelect', 'checkbox']:
            if type(answer) == 'NoneType':
                return []
        return answer

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCommentsFor')
    def getCommentsFor(self, userid):
        """Get a specific user's comments"""
        return self.answers.get(userid, {}).get('comments', None)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAggregateAnswers')
    def getAggregateAnswers(self):
        """Return a mapping of aggregrate answer values,
        suitable for a histogram"""
        if self.getInputType() in ['area', 'text', 'text50', 'text100', 'text150']:
            return {}
        aggregate_answers = {}
        options = self.getAnswerOptions()
        for option in options:
            aggregate_answers[option] = 0
        for k,answer in self.getAnswers().items():
            if answer['value']:
                if isinstance(answer['value'], str):
                    aggregate_answers[answer['value']] += 1
                else:
                    for value in answer['value']:
                        aggregate_answers[value] += 1
        return aggregate_answers

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getPercentageAnswers')
    def getPercentageAnswers(self):
        """Return a mapping of aggregrate answer values,
        suitable for a barchart"""
        max = 0
        aggregate_answers = self.getAggregateAnswers()
        for k,v in aggregate_answers.items():
            if v > max:
                max = v
        pct_aggregate_answers = {}
        for k,v in aggregate_answers.items():
            if v == 0:
                value = 0
            else:
                value = v/float(max)
            pct_aggregate_answers[k] = int(value * 100)
        return pct_aggregate_answers

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getColors')
    def getColors(self, num_options):
        """Return the colors for the barchart"""
        colors = BARCHART_COLORS
        num_colors = len(colors)
        while num_colors < num_options:
            colors = colors + colors
            num_colors = len(colors)
        return colors

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getComments')
    def getComments(self):
        """Return a userid, comments mapping"""
        mlist = []
        for k, v in self.getAnswers().items():
            mapping = {}
            mapping['userid'] = k
            mapping['comments'] = v.get('comments', '')
            mlist.append(mapping)
        return mlist

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAverageAnswer')
    def getAverageAnswer(self, raise_exception="no"):
        """Return an average of numeric answers.  If *raise_exception*
        is passed, then this method will cause an error if any answers
        are not numeric"""
        raise "NotImplemented"

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getHistogram')
    def getHistogram(self):
        """Return an image representing answer data"""
        raise "NotImplemented"

registerType(RegistrationQuestion)