from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.MeetingRegistration.Permissions import *

schema = BaseSchema + Schema((

    StringField('abstract',
        searchable=1,
        required=0,
        accessor="getAbstract",
        edit_accessor="getAbstract",
        mutator="setAbstract",
        widget=TextAreaWidget(
            label="Registration description",
            label_msgid="label_description",
            description="Add a short description of the registration here.",
            description_msgid="help_description",
            rows=5),
        ),

    ))

Actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'string:${object_url}/registration_view',
    'permissions' : (VIEW_CONTENTS_PERMISSION, )
    },
)

class SubRegistration(OrderedBaseFolder):
    """A sub page within a registration"""
    schema = schema
    actions = Actions
    immediate_view = "base_edit"
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = (['Registration Question'])
    archetypes_name = portal_type = "Sub Registration"
    meta_type = "SubRegistration"
    content_icon = "icon_meetingregistration_small.gif"
    include_default_actions = 1
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAbstract')
    def getAbstract(self, **kw):
        return self.Description()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setAbstract')
    def setAbstract(self, val, **kw):
        self.setDescription(val)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isMultipage')
    def isMultipage(self):
        """Return true if there is more than one page in the registration"""
        return True

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getQuestions')
    def getQuestions(self):
        """Return the questions for this part of the registration"""
        questions = self.getFolderListingFolderContents(contentFilter={'portal_type':'RegistrationQuestion',})
        return questions

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'checkCompleted')
    def checkCompleted(self):
        """Return true if this page is completed"""
        # XXX
        return True

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNextPage')
    def getNextPage(self):
        """Return the next page of the registration"""
        parent = self.aq_parent
        pages = parent.getFolderListingFolderContents(contentFilter={'portal_type':'Sub Registration',})
        num_pages = len(pages)
        for i in range(num_pages):
            if pages[i].getId() == self.getId():
                current_page = i
        try:
            return pages[current_page+1]()
        except IndexError:
            # no next page, so registration finished
            parent.setCompletedForUser()
            return parent.exitRegistration()

registerType(SubRegistration)