"""An Organizational Profile type"""

__author__  = 'Daniel J. Vreeman, PT, DPT'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo


try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.OrgProfile.interfaces import IOrgProfile

from Products.ATContentTypes.content.newsitem import ATNewsItem
from Products.ATContentTypes.content.newsitem import finalizeATCTSchema

from Products.validation.config import validation
from Products.validation.validators.SupplValidators import MaxSizeValidator
from Products.validation import V_REQUIRED
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.document import ATDocument
from Products.CMFCore.permissions import View

validation.register(MaxSizeValidator('checkNewsImageMaxSize',
                                     maxsize=zconf.ATNewsItem.max_file_size))

from Products.validation.validators.SupplValidators import MaxSizeValidator


# Copy the ATNewsItemschema (to avoid modifying the original) and append our
# own fields for the images and attachments manager widgets.
FirstOrgProfileSchema = ATNewsItem.schema.copy()
del FirstOrgProfileSchema['title']
del FirstOrgProfileSchema['description']
del FirstOrgProfileSchema['image']
del FirstOrgProfileSchema['imageCaption']
del FirstOrgProfileSchema['text']

OrgProfileSchema = FirstOrgProfileSchema + Schema((

    StringField('title',
        required=1,
        searchable=1,
        default='',
        accessor='getTitle',
        widget=StringWidget(
            label="Organization",
            label_msgid="label_organization",
            description = "Enter the name of the organization.",
            description_msgid="help_organization"),
    ),

   StringField('description',
        required=1,
        searchable=1,
        default='',
        accessor='getDescription',
        widget=TextAreaWidget(
            label="Description of how you are using LOINC",
            label_msgid="label_description",
            description = "Please describe how your organization is using LOINC. This text shows up in the listings of organizations.",
            description_msgid="help_description"),
    ),
    
    TextField('text',
        required = False,
        searchable = True,
        primary = True,
        accessor='getText',
        storage = AnnotationStorage(migrate=True),
        validators = ('isTidyHtmlWithCleanup',),
        #validators = ('isTidyHtml',),
        default_content_type = zconf.ATNewsItem.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATNewsItem.allowed_content_types,
        widget = RichWidget(
            description = "Additional formatted description of the organization. This text does not show up in the listings of organizations.",
            description_msgid = "help_additional_description",
            label = "Additional Description",
            label_msgid = "label_additional_description",
            rows = 3,
            i18n_domain = "plone",
            allow_file_upload = zconf.ATDocument.allow_document_upload)
        ),



    StringField('contactName',
        required=1,
        searchable=1,
        default='',
        accessor='getContactName',
        widget=StringWidget(
            label="Contact Name",
            label_msgid="label_contact_name",
            description = "Enter the full name of the organizational contact.",
            description_msgid="help_contact_name"),
    ),

    StringField('contactTitle',
        required=0,
        searchable=1,
        default='',
        accessor='getContactTitle',
        widget=StringWidget(
            label="Contact Title",
            label_msgid="label_contact_title",
            description = "Enter the contact's title or position within the organization.",
            description_msgid="help_contact_title"),
    ),

    TextField('contactAddress',
        required=1,
        searchable=1,
        default='',
        storage = AnnotationStorage(migrate=True),
        default_output_type = 'text/x-html-safe',
        accessor='getContactAddress',
        widget=TextAreaWidget(
            label="Contact Address",
            label_msgid="label_address",
            description = "Enter address of organizational contact.",
            description_msgid="help_address"),
    ),

    StringField('phone',
        required=1,
        searchable=1,
        default='',
        accessor='getPhone',
        widget=StringWidget(
            label="Contact Phone",
            label_msgid="label_phone",
            description = "Enter a phone number for the organizational contact.",
            description_msgid="help_phone"),
    ),

    StringField('email',
        required=1,
        searchable=1,
        default='',
        accessor='getEmail',
        widget=StringWidget(
            label="Contact Email Address",
            label_msgid="label_email",
            description = "Enter an email address for the organizational contact .",
            description_msgid="help_email"),
    ),

    StringField('url',
        required=0,
        searchable=1,
        default='',
        accessor='getURL',
        widget=StringWidget(
            label="Organizational URL",
            label_msgid="label_URL",
            description = "Enter web address for organization. Include the http://prefix.",
            description_msgid="help_title"),
    ),

    ImageField('image',
        required = False,
        storage = AnnotationStorage(migrate=True),
        languageIndependent = True,
        max_size = zconf.ATNewsItem.max_image_dimension,
        sizes= {'large'   : (768, 768),
                'preview' : (400, 400),
                'mini'    : (200, 200),
                'thumb'   : (128, 128),
                'tile'    :  (64, 64),
                'icon'    :  (32, 32),
                'listing' :  (16, 16),
               },
        validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkNewsImageMaxSize', V_REQUIRED)),
        widget = ImageWidget(
            description = "Will be shown in the organizational listing, and in the item itself. Image will be scaled to a sensible size.",
            description_msgid = "help_news_image",
            label= "Organization Logo",
            label_msgid = "label_org_logo",
            show_content_type = False)
        ),
        
    StringField('imageCaption',
        required = False,
        searchable = True,
        widget = StringWidget(
            description = "",
            description_msgid = "help_image_caption",
            label = "Image Caption",
            label_msgid = "label_image_caption",
            size = 40,
            i18n_domain = "plone")
        ),

    ),)

# Finalise the schema according to ATContentTypes standards. This basically
# moves the Related items and Allow discussion fields to the bottom of the
# form. See ATContentTypes.content.schemata for details.

finalizeATCTSchema(OrgProfileSchema)

# Declare our class to get all the fields from ATNewsItem.

class OrgProfile(ATNewsItem):
    """
    An organizational profile
    """

    # Standard content type setup
    portal_type = meta_type = 'OrgProfile'
    archetype_name = 'Org Profile'
    content_icon = 'OrgProfile.gif'
    schema = OrgProfileSchema
    typeDescription= 'An organizational profile'
    typeDescMsgId  = 'OrgProfile_description_edit'

    # Set up our views - these are available from the 'display' menu
    default_view = 'orgprofile_view'
    immediate_view = 'orgprofile_view'

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Make sure we get all the interface declarations from ATNewsItem,
    # which includes support for ISelectableBrowserDefault to get the
    # 'display' menu to work, IHistoryAware and other standard interfaces.
    # In addition, add INonStructuralFolder to tell Plone that even though
    # this type is technically a folder, it should be treated as a standard
    # content type. This ensures the user doesn't perceive a OrgProfile as
    # a folder.
    __implements__ = ATNewsItem.__implements__ + \
                     ATDocument.__implements__,(IOrgProfile, )
    
     # Get the standard actions (tabs)
    actions = ATNewsItem.actions

    # This method, from ISelectableBrowserDefault, is used to check whether
    # the "Choose content item to use as deafult view" option will be
    # presented. This makes sense for folders, but not for OrgProfile, so
    # always disallow
    def canSetDefaultPage(self):
        return False

    # enable FTP/WebDAV and friends
    PUT = ATNewsItem.PUT
    
    # Stuff from News Item ... dont' know if I need it or not.
    
    security = ClassSecurityInfo()

    security.declareProtected(View, 'tag')
    def tag(self, **kwargs):
          """Generate image tag using the api of the ImageField
          """
          if 'title' not in kwargs:
              kwargs['title'] = self.getImageCaption()
          return self.getField('image').tag(self, **kwargs)
  
    security.declarePrivate('cmf_edit')
    def cmf_edit(self, text, description=None, text_format=None, **kwargs):
          if description is not None:
              self.setDescription(description)
          self.setText(text, mimetype=translateMimetypeAlias(text_format))
          self.update(**kwargs)

    def __bobo_traverse__(self, REQUEST, name):
          """Transparent access to image scales
          """
          if name.startswith('image'):
              field = self.getField('image')
              image = None
              if name == 'image':
                  image = field.getScale(self)
              else:
                  scalename = name[len('image_'):]
                  if scalename in field.getAvailableSizes(self):
                      image = field.getScale(self, scale=scalename)
              if image is not None and not isinstance(image, basestring):
                  # image might be None or '' for empty images
                  return image

          return ATDocument.__bobo_traverse__(self, REQUEST, name)

registerType(OrgProfile)
