## Install.py - Plone QuickInstaller method for product

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.CMFCorePermissions import ManagePortal

from Products.PloneExtendedNavigation.config import *

from StringIO import StringIO

SKIN_NAME = PROJECT_NAME


configlets = (
    {'id' : 'extended_navigation_configuration'
     , 'name' : 'Extended Navigation  configuration'
     , 'action' : 'string:${portal_url}/prefs_manage_navigation'
     , 'category' : 'Products'
     , 'appId' : PROJECT_NAME
     , 'permission' : ManagePortal
     , 'imageUrl' : 'site_icon.gif'
     },)

def install_skins(self, out, skinsdir, product_globals):
    """ Install product skins and register them to portal_skins."""

    print >> out, '* Installing and registering layers from directory %s\n' % skinsdir

    skinstool = getToolByName(self, 'portal_skins')

    # Create the layer in portal_skins
    if skinsdir not in skinstool.objectIds():
        addDirectoryViews(skinstool, skinsdir, product_globals)
        print >> out, 'Added directory view "%s" to portal_skins\n' % skinsdir
    else:
        print >> out, 'Warning: directory view "%s" already added to portal_skins\n' % skinsdir

    # Add the layer to all existing skins
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in path.split(',')]
        try:
            if SKIN_NAME not in path:
                path.insert(path.index('custom') + 1, SKIN_NAME)
        except ValueError:
            if SKIN_NAME not in path:
                path.append(SKIN_NAME)

        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)

def install_tool(self, out, product_globals):
    """ Setup a tool in the Plone site."""

    # Add the tool if it does not already exist.
    bt = getToolByName(self, TOOL_NAME, None)
    if bt is None:
        print >> out, "* Installing %s as %s\n" % (TOOL_PORTAL_NAME, TOOL_NAME)
        portal = getToolByName(self, 'portal_url').getPortalObject()
        addTool = portal.manage_addProduct[PROJECT_NAME].manage_addTool
        addTool(type = TOOL_PORTAL_NAME)

        bt = getToolByName(self, TOOL_NAME)

    # Restrict access to the tool to the Manager only.
    bt.manage_permission('View', ['Manager'])


def addConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Adding configlet %s\n' % conf['id'])
            configTool.registerConfiglet(**conf)

def addConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Adding configlet %s\n' % conf['id'])
            configTool.registerConfiglet(**conf)

def install(self):
    """ Install the product into Plone. """
    out = StringIO()
    portal = getToolByName(self, 'portal_url').getPortalObject()

    install_skins(self, out, SKIN_DIR, GLOBALS)
    install_tool(self, out, GLOBALS)
    addConfiglets(self, out)

    # Add portlet inside 'left_slots' property
    slots = list(portal.getProperty('left_slots'))

    if SLOT_PATH not in slots:
        slots.append(SLOT_PATH)
        portal._updateProperty('left_slots', slots)
        out.write('Added PloneNavigation slot to left_slots property\n')

    return out.getvalue()

def removeConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Removing configlet %s\n' % conf['id'])
            configTool.unregisterConfiglet(conf['id'])

def uninstall(self):
    """ Nothing to do. """
    out=StringIO()
    
    # Remove 'navShow' and 'navOrder' properties from all objects
    # utool = getToolByName(self, 'portal_url')
    # portal = utool.getPortalObject()
    # navtool = getToolByName(self, TOOL_NAME, None)

    # if navtool:
    #    available_objects = navtool.available_objects
    #    list_objects = [x for x in portal.objectValues(available_objects) if x.hasProperty('navShow') or x.hasProperty('navOrder')]
    #    if list_objects:
    #        removeProperties(list_objects)
    #        for obj in list_objects:
    #            sub_list = [x for x in obj.objectValues(available_objects) if x.hasProperty('navShow') or x.hasProperty('navOrder')]
    #            if sub_list:
    #                removeProperties(sub_list)
    
    removeConfiglets(self, out)

    return out.getvalue()

def removeProperties(list_obj):
    """
    remove 'navShow' and 'navOrder' properties from an object list

    """
    for obj in list_obj:
        if obj.hasProperty('navShow'):
            obj.manage_delProperties(['navShow'])
        if obj.hasProperty('navOrder'):
            obj.manage_delProperties(['navOrder'])

    return
