from Products.CMFCore.utils import UniqueObject, getToolByName
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, package_home
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression

from Products.PloneExtendedNavigation.config import *
if HAS_LINGUA_PLONE:
    from Products.PloneLanguageTool.interfaces import ITranslatable

import os

class PloneNavigationTool(UniqueObject, SimpleItem):

    id = TOOL_NAME
    meta_type = TOOL_PORTAL_NAME
    title = TOOL_PORTAL_NAME
    version = TOOL_VERSION
    
    security = ClassSecurityInfo()

    manage_options = (SimpleItem.manage_options)

    # Default properties values
    available_objects = ['Plone Folder', 'Large Plone Folder']
    slot_title = 'Extended navigation'
    root_object_url = ''
    show_all_objects = 0

    #
    # Tool methods
    #

    security.declareProtected(ManagePortal, 'editProperties')
    def editProperties(self,
                       slot_title='',
                       show_all_objects=0,
                       root_object_url='',
                       available_objects=''):
        """
        Edit the plone navigation tool properties

        """

        if slot_title:
            self.slot_title = slot_title
        self.root_object_url = root_object_url
        self.show_all_objects = show_all_objects
        if available_objects:
            self.available_objects = available_objects
        self.REQUEST.RESPONSE.redirect(self.absolute_url() + '/propertiesForm' + '?manage_tabs_message=Tool+Updated.')

    security.declarePublic('getAvailableObjects')
    def getAvailableObjects(self):
        return self.available_objects

    security.declarePublic('getAvailableObjectsSring')
    def getAvailableObjectsString(self):
        """
        Return a string of available objects, to display it inside the textarea field in ZMI
        """
        list = self.available_objects
        result = ''
        for x in list:
            result = result + x + '\n'
        
        return result.strip()

    security.declarePublic('getRootObjectURL')
    def getRootObjectURL(self):
        return self.root_object_url

    security.declarePublic('getSlotTitle')
    def getSlotTitle(self):
        return self.slot_title

    security.declarePublic('isSameURL')
    def isSameURL(self,url1,url2):
        if url1 == url2:
            return 1
        else:
            return 0

    security.declarePublic('isCurrentNav')
    def isCurrentNav(self, object):
        """
        Return 1 if object_url is the current url.
        """

        request = self.REQUEST
        object_url = object.absolute_url()

        object_ids = object.objectIds()

        if object_ids:
            if 'index_html' in object_ids:
                current_url = request.URL
                if current_url[-11:] == '/index_html':
                    return self.isSameURL(current_url[:-11], object_url)
            return self.isSameURL(request.URL1, object_url)
        else:
            return self.isSameURL(request.URL1, object_url)

    security.declarePublic('isCurrentSection')
    def isCurrentSection(self, url):
        """
        Return 1 if url is the current navigation section
        """

        if url == self.REQUEST.URL[:len(url)]:
            return 1
        else:
            return 0

    security.declarePublic('listNavManagement')
    def listNavManagement(self, root_path=None):
        """
        This script return a list of objects contained inside the 'root_path' object.
        If 'root_path' is None, it returns objects contained inside the plone root object.
        """

        available_objects = self.getAvailableObjects()
        
        if root_path is None:
            root_object = self.portal_url.getPortalObject()            
        else:
            root_object = self.restrictedTraverse(root_path)

        root_is_orderable = self.isOrderable(root_object)
        
        if root_is_orderable:
            list = root_object.objectValues(available_objects)
        else:
            ids = root_object.objectIds(available_objects)
            list1=[]
            list2=[]
            for id in ids:
                obj = getattr(root_object, id)
                if obj.hasProperty('navOrder'):
                    list1.append(obj)
                else:
                    list2.append(obj)
            list1.sort(lambda a,b:cmp(a.getProperty('navOrder'), b.getProperty('navOrder')))
            list = list1 + list2
            
        return list

    security.declarePublic('listNav')
    def listNav(self,obj):
        """
        Get objects contained inside 'obj', published, with a navOrder and navShow checked.
        the result list is sort on navOrder property.
        """
        
        available_objects = tuple(self.getAvailableObjects())
        membership_tool = getToolByName(self, 'portal_membership')
        checkPermission = membership_tool.checkPermission
            
        if self.show_all_objects:            
            list = [x for x in obj.objectValues(available_objects) if checkPermission('View', x)]
        else:    
            if self.isOrderable(obj):
                list = [x for x in obj.objectValues(available_objects) if x.getProperty('navShow', '') and checkPermission('View', x)]
            else:
                list = [x for x in obj.objectValues(available_objects) if x.getProperty('navShow', '') and x.getProperty('navOrder', '')  and checkPermission('View', x)]
                list.sort(lambda a,b:cmp(a.getProperty('navOrder'), b.getProperty('navOrder')))

        if HAS_LINGUA_PLONE:
            current_language = self.REQUEST.get('LANGUAGE')
            l = []
            for item in list:
                if not ITranslatable.isImplementedBy(item) or item.Language() == current_language:
                    l.append(item)
            #list = [x for x in list if x.Language()  and x.Language() == current_language or not x.Language()]
            list = l
            
        return list

    security.declarePublic('makePath')
    def makePath(self, url):
        """
        Return a list of dictionnaries : [{'name' : x, 'path' : path}, ...]
        where 'name' is an object id and 'path', the path for this object.
        The url parameter is the absolute_url of the current 'root_path' object
        """

        portal_url = getToolByName(self, 'portal_url').getPortalObject().absolute_url()
        url = url[len(portal_url) + 1:]

        list = url.split('/')
        list_out = []
        path = ''
        for x in list:
            if path == '':
                path = path + x
            else:
                path = path + '/' + x
            list_out.append({'name' : x, 'path' : path})
        return list_out

    security.declarePublic('isOrderable')
    def isOrderable(self, object):
        """
        return 1 if object is orderable
        """
        
        iface_tool = getToolByName(self, 'portal_interface')
        if iface_tool.objectImplements(object, 'Products.CMFPlone.interfaces.OrderedContainer.IOrderedContainer'):
            return 1
        return 0

    

InitializeClass(PloneNavigationTool)
