#------------------------------------------------------------------------------
# Name:         PloneFormMailer.py
# Purpose:      This product integrates Formulator and CMFFormController.
#               The form will be be mailed to a configureable recipient.
#               It is designed to be easily extended and adapted.
#
# Author:       Jens Klein <jens.klein@jensquadrat.com>
# generated by: ArchGenXML Version 0.9.14 http://sf.net/projects/archetypes/
#
# Created:      Sun Jan 25 22:09:22 2004
# Copyright:    (c) 2004-2005 by jens quadrat, Klein & Partner KEG, Austria
# Licence:      GNU General Public Licence (GPL) Version 2 or later
#------------------------------------------------------------------------------
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
from Globals import InitializeClass
from Products.Archetypes.utils import OrderedDict
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent, View
from Products.CMFCore.utils import getToolByName
from Products.DCWorkflow.Scripts import Scripts
from Products.Formulator.Form import ZMIForm
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PythonScripts.PythonScript import manage_addPythonScript
from config import *
from email import Encoders
from email.Header import Header
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEImage import MIMEImage
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
from permissions import SendForm
from schema import PFMschema
from time import strftime
import gpg

if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.public import *

actions=(
    {'action':      'string:${object_url}/formmailer',
     'category':    'object',
     'id':          'view',
     'name':        'View',
     'permissions': ('View',)},
    {'action':      'string:${object_url}/form/manage',
     'category':    'object',
     'id':          'modifyformulatorform',
     'name':        'Modify Form (ZMI)',
     'permissions': ('Modify portal content',)},
)


class PloneFormMailer(BaseContent):
    """This product integrates Formulator and CMFFormController.

    The form will be be mailed to a configureable recipient.
    It is designed to be easily extended and adapted.
    """

    security = ClassSecurityInfo()

    schema  = PFMschema
    actions = actions

    portal_type = meta_type = 'PloneFormMailer'
    archetype_name = 'Form Mailer'   #this name appears in the 'add' box
    content_icon = 'formmailer_icon.png'
    allowed_content_types = []
    global_allow = 1
    filter_content_types = 1
    _at_rename_after_creation = True

    def __init__(self, oid, **kwargs):
        BaseContent.__init__(self, oid, **kwargs)
        self.pt = {}
        self.pt = ZopePageTemplate('pt')
        self.subject_expr = None

        form = ZMIForm('form', 'PFM Formulator', unicode_mode=True)
        setattr(self, 'form', form)
        self.form.manage_addField('fullname', 'Full Name', 'StringField')
        self.form.manage_addField('email', 'e-mail', 'EmailField')
        self.form.manage_addField('message', 'Your Message', 'TextAreaField')

        # Add Scripts tab to Formulator instance (with id 'form') to
        # manange Scripts used in before_script feature.
        self.form.manage_options = ((ZMIForm.manage_options[0],
                                     {'label': 'Scripts', 
                                      'action': 'scripts/manage_main'},) +
                                    ZMIForm.manage_options[1:])
        self.form.scripts = Scripts('scripts')
        scripts = self.form.scripts
        scripts.meta_type = 'PloneFormMailer Scripts'
        if not hasattr(aq_base(scripts), 'DEFAULT_BEFORE_SCRIPT'):
            manage_addPythonScript(scripts, 'DEFAULT_BEFORE_SCRIPT')
            scripts._getOb('DEFAULT_BEFORE_SCRIPT').write(
                DEFAULT_BEFORE_SCRIPT)
        # Some form settings
        self.form.enctype = FORM_ENCTYPE
        self.form.unicode_mode = True

    # special processings #################################
    security.declareProtected(View, 'getButtons')
    def getButtons(self):
        buttons=[]
        for buttondef in self.getFormButtons():
            button= []
            [button.append(s.strip()) for s in buttondef.split('|')]
            buttons.append(button)
        return buttons

    # send_form ###########################################
    security.declarePrivate('get_mail_text')
    def get_mail_text(self, prepend='', append='', **kwargs):
        """Get header and body of e-mail as text (string)
        
        Keyword arguments:
        ua_description -- alternate user agent description used in header and as
        default footer
        """
        
        outer = MIMEMultipart()

        (headerinfo, additional_headers, 
         body) = self.get_header_body_tuple(prepend, append, **kwargs)

        # write header
        for key, value in headerinfo.items():
            outer[key] = value

        # write additional header
        for a in additional_headers:
            key, value = a.split(':', 1)
            outer.add_header(key, value.strip())
        
        # build complete text
        outer.attach(MIMEText(body, _subtype=self.getBodyType() or
                              'html', _charset=self._site_encoding()))

        for attachment in self.get_attachments():
            filename = attachment[0]
            ctype = attachment[1]
            encoding = attachment[2]
            content = attachment[3]

            if ctype is None:
                ctype = 'application/octet-stream'
                 
            maintype, subtype = ctype.split('/', 1)
             
            if maintype == 'text':
                msg = MIMEText(content, _subtype=subtype)
            elif maintype == 'image':
                msg = MIMEImage(content, _subtype=subtype)
            elif maintype == 'audio':
                msg = MIMEAudio(content, _subtype=subtype)
            else:
                msg = MIMEBase(maintype, subtype)
                msg.set_payload(content)
                # Encode the payload using Base64
                Encoders.encode_base64(msg)
                 
            # Set the filename parameter
            msg.add_header('Content-Disposition', 'attachment', filename=filename)
            outer.attach(msg)

        return outer.as_string()

    def get_attachments(self):
        """Return all attachments uploaded in form.
        """
        
        from ZPublisher.HTTPRequest import FileUpload
        from OFS.content_types import guess_content_type

        attachments = []
        
        for field in self.form.get_fields():
            if field.meta_type == 'FileField':
                file = field.validate(field.REQUEST)
                if isinstance(file, FileUpload) and file.filename != '':
                    file.seek(0) # rewind
                    data = file.read()
                    filename = file.filename
                    mimetype, enc = guess_content_type(filename, data, None)
                    attachments.append((filename, mimetype, enc, data))
        return attachments
                    
    security.declarePrivate('get_mail_body')
    def get_mail_body(self, prepend='', append='', hidefields=[], **kwargs):
        """Returns the mail-body with footer.
        """

        if kwargs.has_key('request'):
            request = kwargs['request']
        else:
            request = self.REQUEST

        UA_DESCRIPTION = \
            'PloneFormMailer (send web forms via e-mail using Zope/Plone)'

        if kwargs.has_key('ua_description'):
            UA_DESCRIPTION = kwargs['ua_description']

        if type(hidefields)!=type([]):
            hidefields=[hidefields]

        info = {
            'prepend': prepend,
            'append':  append,
            'useragent': UA_DESCRIPTION,
            'hidefields': hidefields,
        }
        # Update that info dictionary with the values passed in kwargs.
        info.update(kwargs)

        bodyfield = self.getField('body_pt')
        body = bodyfield.get(self, **info)
        
        if kwargs.has_key('encryption'):
            encryption =  kwargs['encryption']
        else:
            encryption = 1

        keyid = self.getGPGKeyId()
        encryption = encryption and keyid

        # check if encryption needed.
        if encryption:
            # XXX :-( TODO
            # check if keyid exists in keyring
            # else if keyserver is given try to get it from keyserver
            # else if public_key is given try to import it
            # else
            # print "!!! PloneFormMailer",self.id,(": gpg encryption failed. "
            #                                      "A key is given but invalid.")
            # if not failed
            gpgobj = gpg.gpg_subprocess()
            bodygpg = gpgobj.encrypt(body, keyid)
            if bodygpg.strip():
                body = bodygpg

        return body

    security.declareProtected(View, 'getFooter')
    def getFooter(self, **kwargs):

        UA_DESCRIPTION = \
            'PloneFormMailer (send web forms by e-mail using Zope/Plone)'

        if kwargs.has_key('ua_description'):
            UA_DESCRIPTION = kwargs['ua_description']

        if self.getBodyFooter():
            return '-- \n%s'% self.getBodyFooter()
        else:
            return '-- \n' + UA_DESCRIPTION + '\n\n'+\
                'brought to you by jens quadrat, Klein & Partner KEG, Austria\n'+\
                'http://jensquadrat.com                      http://plone.org'

    security.declarePrivate('secure_header_line')
    def secure_header_line(self, line):
        nlpos = line.find('\n')
        if nlpos >= 0:
            line = line[:nlpos]
        nlpos = line.find('\r')
        if nlpos >= 0:
            line = line[:nlpos]
        return line
        
    security.declarePrivate('get_header_body_tuple')
    def get_header_body_tuple(self, prepend='', append='',
                              from_addr=None, to_addr=None,
                              subject=None, **kwargs):
        """Return header and body of e-mail as an 3-tuple:
        (header, additional_header, body)

        header is a dictionary, additional header is a list, body is a StringIO

        Keyword arguments:
        ua_description -- alternate user agent description used in header and as
                          default footer
        request -- (optional) alternate request object to use
        """

        if kwargs.has_key('request'):
            request = kwargs['request']
        else:
            request = self.REQUEST

        body = self.get_mail_body(prepend, append, '', **kwargs)
        
        if not from_addr:
            for field in self.form.get_fields():
                if field.id == 'email' and not getattr(field, 'sub_form', None):
                    value = request.get('field_%s' % field.id, '')
                    if value:
                        from_addr = value
        
        nosubject = '(no subject)'                
        subject = subject or self.getSubject() or nosubject
        if subject == nosubject:
            for field in self.form.get_fields():
                if field.id == 'email' and not getattr(field, 'sub_form', None):
                    value = request.get('field_%s' % field.id, '')
                    if value:
                        subject = value

        pprops = getToolByName(self, 'portal_properties')
        site_props = getToolByName(pprops, 'site_properties')
        portal = getToolByName(self, 'portal_url').getPortalObject()
        from_addr = from_addr or \
                    site_props.getProperty('email_from_address') or \
                    portal.getProperty('email_from_address')
        recip_name = self.getRecipientName()
        recip_email = self.getRecipientEmail()

        # if no to_addr and no recip_email specified : use owner adress if possible
        # XXX: recipient_email is required, though!
        if not recip_email and not to_addr:
            ownerinfo = self.getOwner()
            ownerid=ownerinfo.getId()
            pms = getToolByName(self, 'portal_membership')
            userdest = pms.getMemberById(ownerid)
            toemail = userdest.email
            to = '%s <%s>' %(ownerid,toemail)
        else:
            to = to_addr or '%s <%s>' % (recip_name, recip_email)

        # transform subject into mail header encoded string
        msgSubject = str(Header(subject,self._site_encoding()))

        headerinfo = OrderedDict()
        headerinfo['To'] = to
        headerinfo['From'] = from_addr
        headerinfo['Subject'] = msgSubject
        headerinfo['User-Agent'] = 'PloneFormMailer for Zope/Plone by jensquadrat.com'
        headerinfo['MIME-Version'] = '1.0'

        # CC
        cc_recips = filter(None, self.getCCRecipients())
        if cc_recips:
            addrs = ['<%s>' % addr for addr in cc_recips]
            headerinfo['Cc'] = ', '.join(addrs)

        # BCC
        bcc_recips = filter(None, self.getBCCRecipients())
        if bcc_recips:
            addrs = ['<%s>' % addr for addr in bcc_recips]
            headerinfo['Bcc'] = ', '.join(addrs)
            
        for key in headerinfo.keys(): 
            headerinfo[key] = self.secure_header_line(headerinfo[key])

        # return 3-Tuple
        return (headerinfo, self.getAdditionalHeaders(), body)

    security.declareProtected(SendForm, 'send_form')
    def send_form(self, prepend='', append='', **kwargs):
        """Send the form.

        Keyword arguments:
        
        ua_description -- alternate user agent description used in
        header and as default footer.  
        """

        mailtext=self.get_mail_text(prepend, append, **kwargs)

        host = self.MailHost
        host.send(mailtext)


    # translation and encodings
    def _site_encoding(self):
        site_props = self.portal_properties.site_properties
        return site_props.default_charset or 'UTF-8'

    # get scripts list ( modified code from PortalTransport)
    def _get_scripts_list(self):
        res = [('', '-- None --'),]
        parent=aq_base(aq_inner(self)).form
        if hasattr(parent,'scripts'):
            for ob in parent.scripts.objectValues('Script (Python)'):
                res.append((ob.getId(), ob.title_or_id()))
        return DisplayList((res))

    # modified code from PortalTransport
    security.declareProtected(View, 'getBeforeScript')
    def getBeforeScript(self):
        """ Return BeforeScript value """
        # workaround for instances without before_script field
        bsfield = self.getField('before_script')
        if bsfield:
            script = bsfield.get(self)
        else:
            script = None

        if script:
            parent = aq_base(aq_inner(self)).form.scripts
            try:
                return parent._getOb(script)
            except AttributeError:
                pass

        return None

    # modified code from PortalTransport
    security.declarePublic('runBeforeScriptIfNeeded')
    def runBeforeScriptIfNeeded(self):
        """Run a script before form template is rendered.

        This is dependend on whether there *is* a before_script.
        """

        script = self.getBeforeScript()
        if callable(script):
            # fixme: is needed move to try except block?
            script.__of__(self)()

def modify_fti(fti):
    # hide unnecessary tabs (usability enhancement)
    for a in fti['actions']:
        if a['id'] in ('metadata', 'references'):
            a['visible'] = 0
    return fti

InitializeClass(PloneFormMailer)
registerType(PloneFormMailer)
