from Products.CMFCore.utils import UniqueObject 
from OFS.SimpleItem import SimpleItem 
from Globals import InitializeClass 
from LocalFolderFilterType import LocalFolderFilterType

class PloneLocalFolderNGTool (UniqueObject, SimpleItem): 
  """ PloneLocalFolderNGTool """ 
  id = 'PLFNG_tool' 
  meta_type= 'PloneLocalFolderNG Tool' 
  plone_tool = 1 
  
  localFolderFilters = []
  
  def __init__(self):
      """ populate a default localFolderFilter to this tool """

      if len(self.localFolderFilters) == 0:
          self.localFolderFilters.append(LocalFolderFilterType())

  def addLocalFolderFilter(self, **kwargs):
      """ add a localFolderFilter to this tool """
      id = kwargs['id']
      existingFilterIds = self.getLocalFolderFilterIdList()
      if id and not id in existingFilterIds:
          self.localFolderFilters.append(LocalFolderFilterType(kwargs))
          return 1
      else:
          return 0

  def updateLocalFolderFilter(self, **kwargs):
      """ add a localFolderFilter to this tool """
      
      filterToUpdate = None

      id = kwargs.get('id', '')

      for filter in self.localFolderFilters:
          if id == filter.id:
              filterToUpdate = filter
              break
      if filterToUpdate:
          filterToUpdate.set(kwargs)
          return 1
      else:
          return 0

  def dumpLocalFolderFilter(self, filterId):
      """ return the key-value attribute info for a localFolderFilter """
      
      filterToDump = None
      resultList = []

      for filter in self.localFolderFilters:
          if filterId == filter.id:
              filterToDump = filter
              break
      
      if filterToDump:
          resultList.append('hidden_file_prefixes:'+filterToDump.hidden_file_prefixes)
          resultList.append('hidden_file_suffixes:'+filterToDump.hidden_file_suffixes)
          resultList.append('hidden_file_names:'+filterToDump.hidden_file_names)
          resultList.append('hidden_folder_prefixes:'+filterToDump.hidden_folder_prefixes)
          resultList.append('hidden_folder_suffixes:'+filterToDump.hidden_folder_suffixes)
          resultList.append('hidden_folder_names:'+filterToDump.hidden_folder_names)
      
      return resultList

  def getLocalFolderFilterIdList(self):
      """ return the list of id's for the current set of localFolderFilters """
      filterNames = []
      for filter in self.localFolderFilters:
          filterNames.append(filter.id)
      return filterNames     

InitializeClass(PloneLocalFolderNGTool) 