"""An Event type which may contain directly uploaded attachments"""

__author__  = 'Daniel J. Vreeman, PT, DPT'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.RichEvent.interfaces import IRichEvent

from Products.RichEvent.widget import AttachmentsManagerWidget
from Products.RichEvent.widget import ImagesManagerWidget

from Products.ATContentTypes.content.event import ATEvent
from Products.ATContentTypes.content.event import finalizeATCTSchema

from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

# Copy the ATEventschema (to avoid modifying the original) and append our
# own fields for the images and attachments manager widgets.
RichEventSchema = ATEvent.schema.copy() + Schema((
        
        BooleanField('displayImages',
            default=False,
            languageIndependent=0,
            widget=ImagesManagerWidget(
                description="If selected, a list of uploaded images will be "
                             "presented at the bottom of the document to allow "
                             "them to be easily downloaded.",
                description_msgid='RichEvent_help_displayImages',
                i18n_domain='RichEvent',
                label="""Display images download box""",
                label_msgid='RichEvent_label_displayImages',
            ),
        ),

        BooleanField('displayAttachments',
            default=True,
            languageIndependent=0,
            widget=AttachmentsManagerWidget(
                description="If selected, a list of uploaded attachments will be "
                             "presented at the bottom of the event to allow "
                             "for easy downloading",
                description_msgid='RichEvent_help_displayAttachments',
                i18n_domain='RichEvent',
                label="""Display attachments download box""",
                label_msgid='RichEvent_label_displayAttachments',
            ),
        ),

    ),)

# Finalise the schema according to ATContentTypes standards. This basically
# moves the Related items and Allow discussion fields to the bottom of the
# form. See ATContentTypes.content.schemata for details.
finalizeATCTSchema(RichEventSchema)

# Declare our class to be first a folderish object, and second get all the
# fields from ATEvent.

class RichEvent(OrderedBaseFolder, ATEvent):
    """
    An Event which may contain directly uploaded images and attachments
    """

    # Standard content type setup
    portal_type = meta_type = 'RichEvent'
    archetype_name = 'Rich event'
    content_icon = 'RichEvent.gif'
    schema = RichEventSchema
    typeDescription= 'A document which can contain rich text, images and attachments'
    typeDescMsgId  = 'RichEvent_description_edit'


    # Allow images and files to be uploaded into this containerish object
    allowed_content_types = ['ImageAttachment', 'FileAttachment']

    # Set up our views - these are available from the 'display' menu
    default_view = 'richevent_view'
    immediate_view = 'richevent_view'
    #suppl_views = ('richevent_view_preview', 'richevent_view_float')

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Make sure we get all the interface declarations from ATEvent,
    # which includes support for ISelectableBrowserDefault to get the
    # 'display' menu to work, IHistoryAware and other standard interfaces.
    # In addition, add INonStructuralFolder to tell Plone that even though
    # this type is technically a folder, it should be treated as a standard
    # content type. This ensures the user doesn't perceive a RichEvent as
    # a folder.
    __implements__ = OrderedBaseFolder.__implements__ + \
                     ATEvent.__implements__ + \
                     (IRichEvent, INonStructuralFolder,)

    # Get the standard actions (tabs)
    actions = ATEvent.actions

    # This method, from ISelectableBrowserDefault, is used to check whether
    # the "Choose content item to use as deafult view" option will be 
    # presented. This makes sense for folders, but not for RichEvent, so
    # always disallow
    def canSetDefaultPage(self):
        return False

    # enable FTP/WebDAV and friends
    PUT = ATEvent.PUT

registerType(RichEvent)
